@@ -1,5 +1,5 @@
 package MooX::Role::POE::Emitter;
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use Moo::Role;
 
@@ -30,6 +30,17 @@ has 'alias' => (
   default   => sub { "$_[0]" },
 );
 
+around 'set_alias' => sub {
+  my ($orig, $self, $value) = @_;
+
+  if ( $poe_kernel->alias_resolve( $self->session_id ) ) {
+    $self->call( '__emitter_reset_alias', $value );
+    $self->emit( $self->event_prefix . 'alias_set', $value );
+  }
+
+  $self->$orig($value)
+};
+
 has 'event_prefix' => (
   lazy      => 1,
   is        => 'ro',
@@ -145,7 +156,6 @@ sub _start_emitter {
       },
 
       $self => [ qw/
-
         __emitter_notify
 
         __emitter_timer_set
@@ -153,6 +163,7 @@ sub _start_emitter {
 
         __emitter_sigdie
 
+        __emitter_reset_alias
       / ],
 
       (
@@ -317,7 +328,7 @@ sub __decr_ses_refc {
   my ($self, $sess_id) = @_;
   --$self->__emitter_reg_sessions->{$sess_id}->{refc};
   $self->__emitter_reg_sessions->{$sess_id}->{refc} = 0
-    unless $self->__emitter_reg_sessions->{$sess_id}->{refc} > 0
+    unless $self->__emitter_reg_sessions->{$sess_id}->{refc} > 0;
 }
 
 sub __get_ses_refc {
@@ -337,12 +348,13 @@ sub __emitter_drop_sessions {
 
   for my $id (keys %{ $self->__emitter_reg_sessions }) {
     my $count = $self->__get_ses_refc($id);
+    while ( $count-- > 0 ) {
+      $poe_kernel->refcount_decrement(
+        $id, E_TAG
+      )
+    }
 
-    $poe_kernel->refcount_decrement(
-      $id, E_TAG
-    ) while $count-- > 0;
-
-    delete $self->__emitter_reg_sessions->{$id}
+    delete $self->__emitter_reg_sessions->{$id};
   }
 
   1
@@ -364,20 +376,19 @@ sub __emitter_notify {
 
   my %sessions;
 
-  SESSION: for my $regev ('all', $event) {
+  REG: for my $regev ('all', $event) {
     if (exists $self->__emitter_reg_events->{$regev}) {
-      next SESSION unless keys %{ $self->__emitter_reg_events->{$regev} };
-
       $sessions{$_} = 1
-        for values %{ $self->__emitter_reg_events->{$regev} };
+        for keys %{ $self->__emitter_reg_events->{$regev} };
     }
   }
 
   my $meth = $prefix . $event;
 
   ## Our own session will get ->event_prefix . $event first
-  $kernel->call( $_[SESSION], $meth, @args )
-    if delete $sessions{ $_[SESSION]->ID };
+  my $s_id = $_[SESSION]->ID;
+  $kernel->call( $s_id, $meth, @args )
+    if delete $sessions{$s_id};
 
   ## Dispatched to N_$event after our Session has been notified:
   my $eat = $self->_pluggable_process( 'NOTIFY', $event, \@args );
@@ -409,7 +420,6 @@ sub __emitter_start {
   unless ($sender == $kernel) {
     ## Have a parent session.
 
-    ## refcount for this session.
     $kernel->refcount_increment( $s_id, E_TAG );
     $self->__incr_ses_refc( $s_id );
     $self->__reg_ses_id( $s_id );
@@ -417,6 +427,8 @@ sub __emitter_start {
     ## subscribe parent session to all notification events.
     $self->__emitter_reg_events->{all}->{ $s_id } = 1;
 
+    $kernel->post( $s_id, $self->event_prefix . "registered", $self );
+
     ## Detach child session.
     $kernel->detach_myself;
   }
@@ -426,6 +438,11 @@ sub __emitter_start {
   $self
 }
 
+sub __emitter_reset_alias {
+  my ($kernel, $self) = @_[KERNEL, OBJECT];
+  $kernel->alias_set( $_[ARG0] );
+}
+
 sub __emitter_disp_default {
   my ($kernel, $self) = @_[KERNEL, OBJECT];
   my ($event, $args) = @_[ARG0, ARG1];
@@ -501,6 +518,7 @@ sub __shutdown_emitter {
   $self->emit( 'shutdown', @_[ARG0 .. $#_] );
 
   ## Drop sessions and we're spent.
+  $self->call('unsubscribe');
   $self->__emitter_drop_sessions;
 }
 
@@ -524,7 +542,7 @@ sub __emitter_register {
     ## Make sure registered session hangs around
     ##  (until _unregister or shutdown)
     $kernel->refcount_increment( $s_id, E_TAG )
-      unless $s_id eq $self->session_id
+      unless $s_id == $self->session_id
       or $self->__get_ses_refc($s_id);
 
     $self->__incr_ses_refc( $s_id );
@@ -537,14 +555,13 @@ sub __emitter_unregister {
   my ($kernel, $self, $sender) = @_[KERNEL, OBJECT, SENDER];
   my @events = @_[ARG0 .. $#_];
 
-  @events = 'all' unless @events;
+#  @events = 'all' unless @events;
+  @events = keys %{ $self->__emitter_reg_events } unless @events;
 
   my $s_id = $sender->ID;
 
   EV: for my $event (@events) {
     unless (delete $self->__emitter_reg_events->{$event}->{$s_id}) {
-      ## Possible we should just not give a damn?
-      warn "Cannot unregister $event for $s_id -- not registered\n";
       next EV
     }
 
@@ -656,13 +673,15 @@ MooX::Role::POE::Emitter - Pluggable POE event emitter role for cows
 
 =head1 DESCRIPTION
 
+A L<Moo::Role> for a L<POE> Observer Pattern implementation.
+
 Consuming this role gives your class a L<POE::Session> capable of 
 processing events via loaded plugins and/or emitting them to registered 
 "listener" sessions.
 
-This is a L<Moo::Role> for a L<POE> Observer Pattern implementation; 
-it is derived from L<POE::Component::Syndicator> by BINGOS, HINRIK, APOCAL 
-et al, but with more cows ;-) and a few extra features, as well as the 
+It is derived from L<POE::Component::Syndicator> by BINGOS, HINRIK, APOCAL 
+et al, but with more cows ;-) and a few extra features (such as anonymous 
+coderef callbacks; see L</yield>), as well as the 
 faster plugin dispatch system that comes with L<MooX::Role::Pluggable>.
 
 The Emitter role consumes L<MooX::Role::Pluggable>, 
@@ -699,7 +718,8 @@ L</_start_emitter> is called.
 B<alias> specifies the POE::Kernel alias used for our L<POE::Session>; 
 defaults to the stringified object.
 
-Set via B<set_alias>
+Set via B<set_alias> -- if the Emitter is running, a prefixed B<alias_set> 
+event is emitted.
 
 =head4 event_prefix
 
